// Copyright (C) 2013 - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function R = distfun_evrnd ( varargin )
    // Extreme value (Gumbel) random numbers
    //
    // Calling Sequence
    //   R = distfun_evrnd ( mu , sigma )
    //   R = distfun_evrnd ( mu , sigma , [m,n] )
    //   R = distfun_evrnd ( mu , sigma , m , n )
    //
    // Parameters
    //   mu : a matrix of doubles, the average
    //   sigma : a matrix of doubles, the standard deviation. sigma>0.
    //   m : a 1-by-1 matrix of floating point integers, the number of rows of R
    //   n : a 1-by-1 matrix of floating point integers, the number of columns of R
    //   R: a matrix of doubles, the random numbers.
    //
    // Description
    //   Generates random variables from the Extreme value (Gumbel) distribution function.
    //   This is the minimum Gumbel distribution.
    //
    //   Any scalar input argument is expanded to a matrix of doubles of 
    //   the same size as the other input arguments.
    //
    // To get max-Gumbel random numbers:
    //<screen>
    // y = -distfun_evrnd(-mu,sigma,m,n)
    //</screen>
    // 
    // Examples
    // // Set the seed so as to always get the same results.
    // distfun_seedset(1);
    //
    // // Test both mu and sigma expanded
    // computed = distfun_evrnd(1,2,[5,5])
    //
    // // Plot Gumbel random numbers
    // N=1000;
    // x=linspace(-20,5,N);
    // y1=distfun_evpdf(x,0.5,2.);
    // R=distfun_evrnd(0.5,2,10000,1);
    // scf();
    // xtitle("Gumbel distribution","x","Density");
    // plot(x,y1)
    // histplot(20,R);
    // legend(["PDF","Data"],"in_upper_left");
    // 
    // // Compare with CDF
    // R=gsort(R,"g","i");
    // n=size(R,"*");
    // p=distfun_evcdf(R,0.5,2,%t);
    // scf();
    // plot(R,(1:n)'/n,"r-");
    // plot(R,p,"b-");
    // legend(["Empirical","CDF"],"in_upper_left");
    // xtitle("Gumbel distribution","x","P(X<x)");
    //
    // Authors
    //   Copyright (C) 2013 - Michael Baudin

    // Load Internals lib
    path = distfun_getpath (  )
    internallib  = lib(fullfile(path,"macros","internals"))

    [lhs,rhs]=argn()
    apifun_checkrhs ( "distfun_evrnd" , rhs , 2:4 )
    apifun_checklhs ( "distfun_evrnd" , lhs , 0:1 )
    //
    mu = varargin(1)
    sigma = varargin(2)
    //
    // Check type
    apifun_checktype ( "distfun_evrnd" , mu , "mu" , 1 , "constant" )
    apifun_checktype ( "distfun_evrnd" , sigma , "sigma" , 2 , "constant" )
    if ( rhs == 3 ) then
        v = varargin(3)
    end
    if ( rhs == 4 ) then
        m = varargin(3)
        n = varargin(4)
    end
    //
    // Check size : OK
    //
    // Check content
    tiny = number_properties("tiny")
    apifun_checkgreq ( "distfun_evrnd" , sigma , "sigma" , 2 , tiny )
    //
    // Check v, m, n
    distfun_checkvmn ( "distfun_evrnd" , 3 , varargin(3:$) )
    //
    [mu,sigma] = apifun_expandfromsize ( 2 , varargin(1:$) )
    m = size(mu,"r")
    n = size(mu,"c")
    //
    u=distfun_unifrnd(0,1,m,n)
    R=mu+sigma.*log(-log(u))
endfunction

